#version 330
out vec4 frag_color;

in vec2 uv;

uniform int ssaa;
uniform ivec2 face_offset;
uniform sampler2D fill_tex;
uniform float fill_opacity;
uniform sampler2D outline_tex;
uniform vec3 stroke_color;
uniform float stroke_opacity;

const float PI = 3.14159265;

vec4 linear_to_sRGB(vec4 color) {
    bvec3 cutoff = lessThan(color.rgb, vec3(0.0031308));
    vec3 higher = 1.055 * pow(color.rgb, vec3(1.0 / 2.4)) - 0.055;
    vec3 lower = color.rgb * 12.92;
    return vec4(mix(higher, lower, cutoff), color.a);
}

vec4 sRGB_to_linear(vec4 color) {
    bvec3 cutoff = lessThan(color.rgb, vec3(0.04045));
    vec3 higher = pow((color.rgb + 0.055) / 1.055, vec3(2.4));
    vec3 lower = color.rgb / 12.92;
    return vec4(mix(higher, lower, cutoff), color.a);
}

vec4 blend_over(vec4 front, vec4 back) {
    float a = front.a + back.a * (1.0 - front.a);
    
    if (a == 0.0) {
        return vec4(0.0);
    }
    
    vec3 c = front.rgb * front.a + back.rgb * back.a * (1.0 - front.a);
    return vec4(c / a, a);
}

void main() {
    vec4 fill_color = vec4(0.0);
    vec2 resolution = vec2(textureSize(outline_tex, 0));
    
    for (int x = 0; x < ssaa; ++x) {
        for (int y = 0; y < ssaa; ++y) {
            vec2 frag_coord = (floor(uv * resolution) * float(ssaa) + vec2(x, y) + 0.5) / float(ssaa);
            vec4 sample_color = sRGB_to_linear(texelFetch(fill_tex, face_offset + ivec2(frag_coord * float(ssaa)), 0));
            fill_color += vec4(sample_color.rgb * sample_color.a, sample_color.a);
        }
    }
    
    if (fill_color.a != 0.0) {
        fill_color.rgb /= fill_color.a;
    }
    
    fill_color.a /= float(ssaa * ssaa);
    fill_color = linear_to_sRGB(fill_color);
    fill_color.a *= fill_opacity;
    vec4 outline_color = vec4(stroke_color, stroke_opacity);
    outline_color.a *= texture(outline_tex, uv).x;
    frag_color = blend_over(outline_color, fill_color);
}
