#version 330
layout (location = 0) in vec2 a_pos;

out vec2 p;

uniform mat4 mvp;
uniform bool inner_extrusion;
uniform mat4 path_to_screen;
uniform int face_to_draw;
const int NONE = 0;
const int FRONT = 1;
const int BACK = 2;
uniform int extrusion_mode;
const int DIRECTIONAL = 1;
const int VANISHING = 2;
const int CAMERA = 3;
uniform vec2 extrusion_direction;
uniform vec2 extrusion_vanishing_point;
uniform float back_extrusion_amount;
uniform float front_extrusion_amount;

vec4 extrude(vec3 p) {
    // Extrude the path-space position `p` into it's clip-space position
    // p.z ==  0.0: front face
    // p.z == -1.0: back face
    float extrusion_amount = mix(front_extrusion_amount, back_extrusion_amount, -p.z);
    
    switch (extrusion_mode) {
        case DIRECTIONAL:
            return vec4(p.xy + extrusion_direction * extrusion_amount, p.z, 1.0);
        case VANISHING:
            return vec4(mix(p.xy, extrusion_vanishing_point, extrusion_amount), p.z, 1.0);
        case CAMERA:
            return path_to_screen * vec4(p.xy, -extrusion_amount, 1.0);
    }
}

void main() {
    p = a_pos;
    vec4 pos = extrude(vec3(a_pos, face_to_draw == FRONT ? 0.0 : -1.0));
    pos.z = extrusion_mode == CAMERA ? 0.5 - 0.5 * pos.z : inner_extrusion ? -1.0 : 0.0;
    gl_Position = mvp * pos;
    gl_Position.z = 0.0;
}
