#version 330
out vec4 frag_color;

in vec2 normal;
flat in float depth;

uniform samplerBuffer side_fill_color_buffer;
uniform bool interpolate_gradient;

const float PI = 3.1415927;

vec4 linear_to_sRGB(vec4 color) {
    bvec3 cutoff = lessThan(color.rgb, vec3(0.0031308));
    vec3 higher = 1.055 * pow(color.rgb, vec3(1.0 / 2.4)) - 0.055;
    vec3 lower = color.rgb * 12.92;
    return vec4(mix(higher, lower, cutoff), color.a);
}

vec4 sRGB_to_linear(vec4 color) {
    bvec3 cutoff = lessThan(color.rgb, vec3(0.04045));
    vec3 higher = pow((color.rgb + 0.055) / 1.055, vec3(2.4));
    vec3 lower = color.rgb / 12.92;
    return vec4(mix(higher, lower, cutoff), color.a);
}

void main() {
    // vec2 normal = normalize(normal);
    float angle = mod(atan(normal.y, normal.x), 2.0 * PI);
    int n = textureSize(side_fill_color_buffer);
    vec4 stop2;
    int i2 = -1;
    
    for (int i = 0; i < n; ++i) {
        stop2 = texelFetch(side_fill_color_buffer, i);
        
        if (angle <= stop2.a) {
            i2 = i;
            break;
        }
    }
    
    vec4 stop1;
    
    if (i2 == -1) {
        stop1 = texelFetch(side_fill_color_buffer, n - 1);
        stop2 = texelFetch(side_fill_color_buffer, 0);
        stop2.a += 2.0 * PI;
    } else {
        int i1 = i2 - 1;
        stop1 = texelFetch(side_fill_color_buffer, (i1 + n) % n);
        
        if (i1 == -1) {
            stop1.a -= 2.0 * PI;
        }
    }
    
    float t = stop2.a != stop1.a
        ? (angle - stop1.a) / (stop2.a - stop1.a)
        : 0.0;
    t = interpolate_gradient ? smoothstep(0.0, 1.0, t) : round(t);
    stop1.rgb = sRGB_to_linear(vec4(stop1.rgb, 1.0)).rgb;
    stop2.rgb = sRGB_to_linear(vec4(stop2.rgb, 1.0)).rgb;
    vec3 color = mix(stop1.rgb, stop2.rgb, t);
    frag_color = linear_to_sRGB(vec4(color, 1.0));
    gl_FragDepth = depth;
}
