#version 330
layout (location = 0) in vec3 a_position;
layout (location = 1) in vec2 a_normal;
layout (location = 2) in float a_z;

uniform int extrusion_mode;
const int DIRECTIONAL = 1;
const int VANISHING = 2;
const int CAMERA = 3;
uniform vec2 extrusion_direction;
uniform vec2 extrusion_vanishing_point;
uniform float back_extrusion_amount;
uniform float front_extrusion_amount;
uniform mat4 path_to_screen;
uniform mat4 mvp;

out vec2 normal;
flat out float depth;

vec4 extrude(vec3 p, out float weight) {
    // Extrude the path-space position `p` into it's clip-space position
    // p.z ==  0.0: front face
    // p.z == -1.0: back face
    float extrusion_amount = mix(front_extrusion_amount, back_extrusion_amount, -p.z);
    
    switch (extrusion_mode) {
        case DIRECTIONAL:
            weight = 1.0;
            return vec4(p.xy + extrusion_direction * extrusion_amount, p.z, 1.0);
        case VANISHING:
            weight = 1.0 - extrusion_amount;
            return vec4(mix(p.xy, extrusion_vanishing_point, extrusion_amount), p.z, 1.0);
        case CAMERA:
            weight = 1.0;
            return path_to_screen * vec4(p.xy, -extrusion_amount, 1.0);
    }
}

void main() {
    float w;
    vec4 p = extrude(a_position, w);
    p.z = a_z * p.w;
    gl_Position = mvp * p;
    gl_Position.z = 0.0;
    normal = a_normal * w;
    depth = -a_z;
}
